from __future__ import annotations

from .dan.BornholmskBitextMining import *
from .kat.TbilisiCityHallBitextMining import *
from .multilingual.BibleNLPBitextMining import *
from .multilingual.BUCCBitextMining import *
from .multilingual.BUCCBitextMiningFast import *
from .multilingual.DiaBLaBitextMining import *
from .multilingual.FloresBitextMining import *
from .multilingual.IN22ConvBitextMining import *
from .multilingual.IN22GenBitextMining import *
from .multilingual.IndicGenBenchFloresBitextMining import *
from .multilingual.IWSLT2017BitextMinig import *
from .multilingual.NorwegianCourtsBitextMining import *
from .multilingual.NTREXBitextMining import *
from .multilingual.RomaTalesBitextMining import *
from .multilingual.TatoebaBitextMining import *
from .srn.SRNCorpusBitextMining import *
from .vie.VieMedEVBitextMining import *
